"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = undefined;

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _objectWithoutProperties2 = require("babel-runtime/helpers/objectWithoutProperties");

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _extends2 = require("babel-runtime/helpers/extends");

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * ApiClient "interface". Wraps the given apiClient as a singleton object.
 * Given apiClient must implement fetch().
 */
var ApiClient = function () {
	function ApiClient(apiClient) {
		var _this = this;

		var lang = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "en";
		var translations = arguments[2];
		(0, _classCallCheck3.default)(this, ApiClient);

		this.flushCache = function () {
			_this.cache = {};
		};

		this.apiClient = apiClient;
		this.on = {};
		this.lang = lang;
		this.translations = translations;
		this.cache = {};
	}

	/**
  * Implementing apiClient must return a promise that passes the raw response as 1st arg.
  * @param path URL for GET.
  * @param query Object, where keys are param names and values are param values.
  * @returns a Promise.
  */


	(0, _createClass3.default)(ApiClient, [{
		key: "fetchRaw",
		value: function fetchRaw(path, query, options) {
			var _this2 = this;

			return this.apiClient.fetch(path, (0, _extends3.default)({ lang: this.lang }, query || {}), options).catch(function () {
				throw new Error(_this2.translations[_this2.lang].RequestFailed);
			});
		}
	}, {
		key: "fetch",
		value: function fetch(path, query) {
			var _this3 = this;

			var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
			var _options$failSilently = options.failSilently,
			    failSilently = _options$failSilently === undefined ? false : _options$failSilently,
			    fetchOptions = (0, _objectWithoutProperties3.default)(options, ["failSilently"]);

			return this.fetchRaw(path, query, fetchOptions).then(function (response) {
				if (!failSilently && response.status >= 400) {
					throw new Error("Request failed");
				}
				return response.json();
			}).catch(function () {
				if (_this3.cache[path]) delete _this3.cache[path][_this3.getCacheKey(query, options)];
				throw new Error(_this3.translations[_this3.lang].RequestFailed);
			});
		}
	}, {
		key: "getCacheKey",
		value: function getCacheKey(query, options) {
			return (0, _stringify2.default)(query) + (0, _stringify2.default)(options);
		}
	}, {
		key: "fetchCached",
		value: function fetchCached(path, query, options) {
			var cacheKey = this.getCacheKey(query, options);
			if (!this.cache[path]) this.cache[path] = {};
			this.cache[path][cacheKey] = this.cache[path].hasOwnProperty(cacheKey) ? this.cache[path][cacheKey] : this.fetch(path, query, options);
			return this.cache[path][cacheKey];
		}
	}, {
		key: "invalidateCachePath",
		value: function invalidateCachePath(path) {
			delete this.cache[path];
			if (this.on[path]) {
				this.on[path].forEach(function (callback) {
					return callback();
				});
			}
		}

		//TODO on invalidation callbacks

	}, {
		key: "invalidateCachePathQuery",
		value: function invalidateCachePathQuery(path, query) {
			if (this.cache[path]) delete this.cache[path][query];
		}
	}, {
		key: "onCachePathInvalidation",
		value: function onCachePathInvalidation(path, callback) {
			if (!this.on[path]) this.on[path] = [];

			this.on[path].push(callback);
		}
	}, {
		key: "removeOnCachePathInvalidation",
		value: function removeOnCachePathInvalidation(path, callback) {
			if (this.on[path]) {
				this.on[path] = this.on[path].filter(function (fn) {
					return fn !== callback;
				});
			}
		}
	}, {
		key: "setLang",
		value: function setLang(lang) {
			this.lang = lang;
			this.flushCache();
		}
	}]);
	return ApiClient;
}();

exports.default = ApiClient;