"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = undefined;

var _extends2 = require("babel-runtime/helpers/extends");

var _extends3 = _interopRequireDefault(_extends2);

var _getPrototypeOf = require("babel-runtime/core-js/object/get-prototype-of");

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require("babel-runtime/helpers/inherits");

var _inherits3 = _interopRequireDefault(_inherits2);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _class;

var _react = require("react");

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _VirtualSchemaField = require("../VirtualSchemaField");

var _VirtualSchemaField2 = _interopRequireDefault(_VirtualSchemaField);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

window.PropTypes = _propTypes2.default;

var conditionPropType = {
	regexp: _propTypes2.default.string,
	length: _propTypes2.default.number,
	reverse: _propTypes2.default.bool
};

var rulePropType = _propTypes2.default.shape({
	transformations: _propTypes2.default.object.isRequired,
	conditions: function conditions(props, propName, componentName) {
		return (0, _keys2.default)(props[propName]).every(function (prop) {
			return _propTypes2.default.checkPropTypes(conditionPropType, props[propName][prop], propName, componentName);
		});
	}
}).isRequired;

/**
 * Changes formData on onChange event according to conditional rules.
 * uiSchema = {
 *   "ui:options": {
 *     rules: { // Can also be an array of rules
 *       transformations: {
 *        <field name to change if conditions met>: <new value>
 *       },
 *       conditions: {
 *         <conditional field name>: { // Note that regexp and length must both pass for this rule to pass.
 *           regexp: <string> <if the conditional field value matches this regexp, this condition will pass>
 *           length: <number> <if the conditional field value length is equal or more, this conndition will pass>
 *           reverse: <boolean> <Reverses the passing value>
 *         }
 *       }
 *     }
 *   }
 * }
 */

var ConditionalOnChangeField = (0, _VirtualSchemaField2.default)(_class = function (_Component) {
	(0, _inherits3.default)(ConditionalOnChangeField, _Component);

	function ConditionalOnChangeField() {
		(0, _classCallCheck3.default)(this, ConditionalOnChangeField);
		return (0, _possibleConstructorReturn3.default)(this, (ConditionalOnChangeField.__proto__ || (0, _getPrototypeOf2.default)(ConditionalOnChangeField)).apply(this, arguments));
	}

	(0, _createClass3.default)(ConditionalOnChangeField, [{
		key: "onChange",
		value: function onChange(formData) {
			var _this2 = this;

			var _getUiOptions = this.getUiOptions(),
			    rules = _getUiOptions.rules;

			(Array.isArray(rules) ? rules : [rules]).forEach(function (_ref) {
				var conditions = _ref.conditions,
				    transformations = _ref.transformations;

				var passes = true;

				// If transforming field is updated, do nothing
				// (If a transforming field is also a condition field, the field would take the conditional value always when updated).
				if ((0, _keys2.default)(transformations).some(function (field) {
					return formData && formData[field] && formData[field] !== _this2.props.formData[field];
				})) {
					return;
				}

				// If condition fields didn't change, do nothing.
				if ((0, _keys2.default)(conditions).every(function (field) {
					return formData && formData[field] && formData[field] === _this2.props.formData[field];
				})) {
					return;
				}

				for (var field in conditions) {
					var condition = conditions[field];
					var regexp = condition.regexp,
					    length = condition.length,
					    reverse = condition.reverse;

					if (passes && regexp !== undefined) {
						var _regexp = new RegExp(condition.regexp.replace("%default", _this2.props.schema.properties[field].default));
						var result = ("" + formData[field]).match(_regexp);
						passes = reverse ? !result : result;
					}
					if (passes && length !== undefined) {
						var _result = (formData[field] || []).length >= length;
						passes = reverse ? !_result : _result;
					}
					if (!passes) break;
				}
				if (passes) {
					var formDataChange = {};
					if (transformations) for (var transformField in transformations) {
						formDataChange[transformField] = transformations[transformField];
					}
					formData = (0, _extends3.default)({}, formData, formDataChange);
				}
			});
			this.props.onChange(formData);
		}
	}], [{
		key: "getName",
		value: function getName() {
			return "ConditionalOnChangeField";
		}
	}]);
	return ConditionalOnChangeField;
}(_react.Component)) || _class;

ConditionalOnChangeField.propTypes = {
	uiSchema: _propTypes2.default.shape({
		"ui:options": _propTypes2.default.shape({
			rules: _propTypes2.default.oneOfType([rulePropType, _propTypes2.default.arrayOf(rulePropType)]).isRequired
		}).isRequired,
		uiSchema: _propTypes2.default.object
	}).isRequired,
	schema: _propTypes2.default.shape({
		type: _propTypes2.default.oneOf(["object"])
	}).isRequired,
	formData: _propTypes2.default.object.isRequired
};
exports.default = ConditionalOnChangeField;