"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.computeUiSchema = exports.default = exports.operationPropType = exports.rulePropType = undefined;

var _extends2 = require("babel-runtime/helpers/extends");

var _extends3 = _interopRequireDefault(_extends2);

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _getPrototypeOf = require("babel-runtime/core-js/object/get-prototype-of");

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require("babel-runtime/helpers/inherits");

var _inherits3 = _interopRequireDefault(_inherits2);

var _class;

var _react = require("react");

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _VirtualSchemaField = require("../VirtualSchemaField");

var _VirtualSchemaField2 = _interopRequireDefault(_VirtualSchemaField);

var _deepmerge = require("deepmerge");

var _deepmerge2 = _interopRequireDefault(_deepmerge);

var _utils = require("../../utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var rulePropType = exports.rulePropType = _propTypes2.default.oneOfType([_propTypes2.default.shape({
	container: _propTypes2.default.string,
	field: _propTypes2.default.string.isRequired,
	regexp: _propTypes2.default.string,
	valueIn: _propTypes2.default.arrayOf(_propTypes2.default.string),
	valueLengthLessThan: _propTypes2.default.number
}), _propTypes2.default.oneOf(["isAdmin", "isEdit"])]);

var operationPropType = exports.operationPropType = _propTypes2.default.shape({
	type: _propTypes2.default.oneOf(["merge", "wrap"]),
	uiSchema: _propTypes2.default.object.isRequired
});

var casePropType = _propTypes2.default.shape({
	rules: _propTypes2.default.oneOfType([rulePropType, _propTypes2.default.arrayOf(rulePropType)]),
	operations: _propTypes2.default.oneOfType([operationPropType, _propTypes2.default.arrayOf(operationPropType)])
});

/**
 * Transforms uiSchema according to conditional cases. Cases are not mutually exclusive.
 * uiSchema = {"ui:options": {
 *	cases: [
 *		rules: [
 *			{
 *				regexp: The regexp that the field must match.
 *				valueIn: An array that the field must be in.
 *				field: The field that the rule is being checked on (can be a JSON pointer or a field name)
 *			}
 *		],
 *		operations: [ // All the operations are performed if all rules pass.
 *			{
 *				type: "merge" or "wrap" "replace". "merge" deeply merges the conditional uiSchema to the nested uiSchema.
 *				                                   "wrap" sets the nested uiSchema inside conditional uiSchema.
 *				                                   "replace" (default) replaces nested uiSchema with conditional uiSchema.
 *				uiSchema: conditional uiSchema to use.
 *			}
 *		],
 *		else: [<same as operations>] // Operations performed if the rules do not pass.
 *		cache: <boolean> true by default. If you know the cases will cache, set this to false.
 *		       There will be a performance penalty on valueIn rule, as they are not indexed without caching.
 *	]
 * }
 * uiSchema: nested uiSchema
 * }
 */

var ConditionalUiSchemaField = (0, _VirtualSchemaField2.default)(_class = function (_Component) {
	(0, _inherits3.default)(ConditionalUiSchemaField, _Component);

	function ConditionalUiSchemaField() {
		var _ref;

		var _temp, _this, _ret;

		(0, _classCallCheck3.default)(this, ConditionalUiSchemaField);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, (_ref = ConditionalUiSchemaField.__proto__ || (0, _getPrototypeOf2.default)(ConditionalUiSchemaField)).call.apply(_ref, [this].concat(args))), _this), _this.cache = {}, _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
	}

	(0, _createClass3.default)(ConditionalUiSchemaField, [{
		key: "getStateFromProps",
		value: function getStateFromProps(props) {
			var _this2 = this;

			var _getUiOptions = this.getUiOptions(),
			    _getUiOptions$cases = _getUiOptions.cases,
			    cases = _getUiOptions$cases === undefined ? [] : _getUiOptions$cases,
			    _getUiOptions$cache = _getUiOptions.cache,
			    cache = _getUiOptions$cache === undefined ? true : _getUiOptions$cache,
			    arrayMerge = _getUiOptions.arrayMerge;

			var uiSchema = props.uiSchema;


			var computedUiSchema = uiSchema;
			(Array.isArray(cases) ? cases : [cases]).forEach(function (_ref2, idx) {
				var _ref2$rules = _ref2.rules,
				    rules = _ref2$rules === undefined ? [] : _ref2$rules,
				    _ref2$operations = _ref2.operations,
				    operations = _ref2$operations === undefined ? [] : _ref2$operations,
				    _else = _ref2.else,
				    _arrayMerge = _ref2.arrayMerge;

				var passes = void 0;
				if (cache) {
					if (!_this2.cache[idx]) {
						_this2.cache[idx] = {};
					}
					var check = (0, _utils.checkRules)(rules, props, _this2.cache[idx]);
					var _cache = check.cache;

					passes = check.passes;
					_this2.cache = _cache;
				} else {
					passes = (0, _utils.checkRules)(rules, props);
				}

				if (passes) {
					computedUiSchema = computeUiSchema(computedUiSchema, operations, _arrayMerge || arrayMerge);
				} else if (_else) {
					computedUiSchema = computeUiSchema(computedUiSchema, _else, _arrayMerge || arrayMerge);
				}
			});

			return { uiSchema: computedUiSchema };
		}
	}], [{
		key: "getName",
		value: function getName() {
			return "ConditionalUiSchemaField";
		}
	}]);
	return ConditionalUiSchemaField;
}(_react.Component)) || _class;

ConditionalUiSchemaField.propTypes = {
	uiSchema: _propTypes2.default.shape({
		"ui:options": _propTypes2.default.shape({
			cases: _propTypes2.default.oneOfType([casePropType, _propTypes2.default.arrayOf(casePropType)]),
			cache: _propTypes2.default.boolean
		})
	}),
	schema: _propTypes2.default.shape({
		type: _propTypes2.default.oneOf(["object", "array"])
	}).isRequired,
	formData: _propTypes2.default.oneOfType([_propTypes2.default.object, _propTypes2.default.array]).isRequired
};
exports.default = ConditionalUiSchemaField;
var computeUiSchema = exports.computeUiSchema = function computeUiSchema(uiSchema, operations) {
	var arrayMerge = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "replace";

	if (!operations) return uiSchema;
	return (Array.isArray(operations) ? operations : [operations]).reduce(function (_uiSchema, op) {
		switch (op.type) {
			case "merge":
				uiSchema = (0, _deepmerge2.default)(uiSchema, op.uiSchema, { arrayMerge: arrayMerge === "replace" ? function (a1, a2) {
						return a2;
					} : function (a1, a2) {
						return [].concat((0, _toConsumableArray3.default)(a1), (0, _toConsumableArray3.default)(a2));
					} });
				break;
			case "wrap":
				uiSchema = (0, _extends3.default)({}, op.uiSchema, { uiSchema: uiSchema });
				break;
			case "replace":
			default:
				uiSchema = op.uiSchema;
		}
		return uiSchema;
	}, uiSchema);
};