"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = undefined;

var _slicedToArray2 = require("babel-runtime/helpers/slicedToArray");

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _defineProperty2 = require("babel-runtime/helpers/defineProperty");

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends10 = require("babel-runtime/helpers/extends");

var _extends11 = _interopRequireDefault(_extends10);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _getPrototypeOf = require("babel-runtime/core-js/object/get-prototype-of");

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require("babel-runtime/helpers/inherits");

var _inherits3 = _interopRequireDefault(_inherits2);

var _class;

exports.getPropsForFields = getPropsForFields;

var _react = require("react");

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _immutabilityHelper = require("immutability-helper");

var _immutabilityHelper2 = _interopRequireDefault(_immutabilityHelper);

var _utils = require("react-jsonschema-form/lib/utils");

var _utils2 = require("../../utils");

var _VirtualSchemaField = require("../VirtualSchemaField");

var _VirtualSchemaField2 = _interopRequireDefault(_VirtualSchemaField);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Makes it possible to extract fields from object schema and
 * make them act like a nested schema without touching the form data structure.
 *
 * uiSchema = {
 *  title: <string>,
 *  ui:options: {
 *    nests: {
 *      fieldName: {
 *       fields: [<string>],
 *       title: <string>,
 *      }
 *    },
 *    uiSchema: <uiSchema>
 *  }
 * }
 *
 * Example usage:
 *
 * schema = {
 *  "type": "object",
 *  "properties": {
 *    "inner_1" { "type": "string" },
 *    "inner_2": { "type": "string" },
 *    "innerOfInner_1": { "type": "string" },
 *    "innerOfInner_2": { "type": "string" },
 *    "secondInner_1" { "type": "string" },
 *    "secondInner_2": { "type": "string" },
 *    "outer_2": { "type": "string" }
 *  }
 *
 *  uiSchema = {
 *    "ui:field": "nest",
 *    "ui:options": {
 *      "nests": {
 *        "inner": {
 *          "fields": ["inner_1", "inner_2"],
 *          "title": "title of inner",
 *          "uiSchema": {
 *            "ui:field": "someField"
 *            "inner_1": {
 *              "ui:field": "someField2"
 *            }
 *          }
 *        },
 *        "secondInner": {
 *          "fields": ["secondInner_1", "secondInner_2"],
 *          "uiSchema": {
 *            "ui:field": "someField3"
 *          }
 *        }
 *      },
 *      "uiSchema": {
 *        "ui:field": "container"
 *      }
 *  }
 *
 *  would make the schemas look like this:
 *
 *  schema = {
 *    "type": "object",
 *    "properties": {
 *      "inner": {
 *        "type": "object",
 *        "title": "title of inner",
 *        "properties": {
 *          "inner_1": {
 *            "type": "string"
 *          },
 *          "inner_2": {
 *            "type": "string"
 *          }
 *        }
 *      },
 *      "inner2": {
 *        "type: "object",
 *        "properties": {
 *          "secondInner_1": {
 *            "type": "string"
 *          },
 *          "secondInner_2": {
 *            "type": "string"
 *          }
 *        }
 *      },
 *    }
 *  }
 *
 *  uiSchema = {
 *    "ui:field": "container",
 *    "inner": {
 *      "ui:field": "someField"
 *      "inner_1": {
 *        "ui:field": "someField2"
 *      }
 *    },
 *    "inner2": {
 *      "ui:field": "someField3"
 *    }
 *  }
 *
 */
var NestField = (0, _VirtualSchemaField2.default)(_class = function (_Component) {
	(0, _inherits3.default)(NestField, _Component);

	function NestField() {
		(0, _classCallCheck3.default)(this, NestField);
		return (0, _possibleConstructorReturn3.default)(this, (NestField.__proto__ || (0, _getPrototypeOf2.default)(NestField)).apply(this, arguments));
	}

	(0, _createClass3.default)(NestField, [{
		key: "getStateFromProps",
		value: function getStateFromProps(props) {
			var options = this.getUiOptions();

			var schema = props.schema,
			    uiSchema = props.uiSchema,
			    idSchema = props.idSchema,
			    errorSchema = props.errorSchema,
			    formData = props.formData,
			    registry = props.registry;
			var nests = options.nests,
			    buttonsNest = options.buttonsNest;

			var buttons = (0, _utils2.getUiOptions)(uiSchema).buttons;
			var uiButtons = uiSchema["ui:buttons"];

			var nestedPropsMap = {};

			var nestNames = (0, _keys2.default)(nests).reduce(function (names, name) {
				names[name] = true;
				return names;
			}, {});

			(0, _keys2.default)(nests).forEach(function (wrapperFieldName) {
				var nest = nests[wrapperFieldName];
				var nestedProps = getPropsForFields({ schema: schema, uiSchema: uiSchema, idSchema: idSchema, errorSchema: errorSchema, formData: formData, registry: registry }, nests[wrapperFieldName].fields, nest.title);
				nestedPropsMap[wrapperFieldName] = nestedProps;

				schema = (0, _extends11.default)({}, schema, { properties: (0, _extends11.default)({}, schema.properties, (0, _defineProperty3.default)({}, wrapperFieldName, nestedProps.schema)) });

				var nestedIdSchema = nestedProps.idSchema;
				nest.fields.forEach(function (field) {
					if (nestNames[field]) {
						nestedIdSchema[field] = nestedPropsMap[field].idSchema;
					}
				});
				idSchema = (0, _extends11.default)({}, idSchema, (0, _defineProperty3.default)({}, wrapperFieldName, nestedIdSchema));

				errorSchema = (0, _extends11.default)({}, errorSchema, (0, _defineProperty3.default)({}, wrapperFieldName, nestedProps.errorSchema));

				formData = (0, _extends11.default)({}, formData, (0, _defineProperty3.default)({}, wrapperFieldName, nestedProps.formData));

				uiSchema = (0, _extends11.default)({}, uiSchema, (0, _defineProperty3.default)({}, wrapperFieldName, (0, _extends11.default)({}, nestedProps.uiSchema, nests[wrapperFieldName] ? nests[wrapperFieldName].rootUiSchema : {} || {}, uiSchema[wrapperFieldName] || {})));
				if (nest.rootUiSchema) {
					uiSchema = (0, _extends11.default)({}, uiSchema, (0, _defineProperty3.default)({}, wrapperFieldName, (0, _extends11.default)({}, nest.rootUiSchema || {}, uiSchema[wrapperFieldName] || {})));
				}

				nests[wrapperFieldName].fields.forEach(function (fieldName) {
					[schema.properties, errorSchema, idSchema, formData, uiSchema].forEach(function (container) {
						delete container[fieldName];
					});
					if (schema.required) schema.required = schema.required.filter(function (requiredField) {
						return requiredField !== fieldName;
					});
				});
			});

			if (buttonsNest && nests[buttonsNest]) {
				if (uiButtons) {
					uiSchema = (0, _defineProperty3.default)({
						"ui:buttons": undefined
					}, buttonsNest, (0, _extends11.default)({}, uiSchema[buttonsNest] || [], {
						"ui:buttons": [].concat((0, _toConsumableArray3.default)(uiSchema["ui:buttons"]))
					}));
				}
				if (buttons) {
					var nestOptions = (0, _utils2.getUiOptions)(uiSchema[buttonsNest]);
					var _buttons = nestOptions.buttons || [];
					uiSchema = (0, _extends11.default)({}, uiSchema, (0, _defineProperty3.default)({
						"ui:options": (0, _extends11.default)({}, uiSchema["ui:options"], {
							buttons: undefined
						})
					}, buttonsNest, (0, _extends11.default)({}, uiSchema[buttonsNest], {
						"ui:options": (0, _extends11.default)({}, nestOptions, {
							buttons: [].concat((0, _toConsumableArray3.default)(_buttons), (0, _toConsumableArray3.default)(buttons))
						})
					})));
				}
			}

			var _uiSchema3 = uiSchema,
			    _uiSchema3$uiOrder = _uiSchema3["ui:order"],
			    order = _uiSchema3$uiOrder === undefined ? [] : _uiSchema3$uiOrder;

			var dictionarifiedOrder = order.reduce(function (dict, field, idx) {
				dict[field] = idx;
				return dict;
			}, {});
			var splices = [];
			if (order) {
				(0, _keys2.default)(nests).forEach(function (nestName) {
					var fields = nests[nestName].fields;

					fields.forEach(function (field) {
						var idx = dictionarifiedOrder[field];
						if (idx !== undefined) {
							splices.push([idx, 1]);
						}
					});
				});
				if (splices.length) uiSchema = (0, _immutabilityHelper2.default)(uiSchema, { "ui:order": { $splice: splices.sort(function (_ref, _ref2) {
							var _ref4 = (0, _slicedToArray3.default)(_ref, 1),
							    idx = _ref4[0];

							var _ref3 = (0, _slicedToArray3.default)(_ref2, 1),
							    _idx = _ref3[0];

							return _idx - idx;
						}) } });
			}

			return { schema: schema, uiSchema: uiSchema, idSchema: idSchema, errorSchema: errorSchema, formData: formData };
		}
	}, {
		key: "onChange",
		value: function onChange(formData) {
			var _getUiOptions = this.getUiOptions(),
			    nests = _getUiOptions.nests;

			(0, _keys2.default)(nests).reverse().forEach(function (nestName) {
				if (formData[nestName]) {
					(0, _keys2.default)(formData[nestName]).forEach(function (prop) {
						if (formData[nestName].hasOwnProperty(prop)) formData = (0, _extends11.default)({}, formData, (0, _defineProperty3.default)({}, prop, formData[nestName][prop]));
					});
					formData = (0, _utils2.immutableDelete)(formData, nestName);
				}
			});

			this.props.onChange(formData);
		}
	}], [{
		key: "getName",
		value: function getName() {
			return "NestField";
		}
	}]);
	return NestField;
}(_react.Component)) || _class;

NestField.propTypes = {
	uiSchema: _propTypes2.default.shape({
		"ui:options": _propTypes2.default.shape({
			"nests": function nests(props, propName, componentName) {
				for (var optionProp in props["ui:options"]) {
					var prop = props["ui:options"][optionProp];
					if (!prop.fields || !Array.isArray(prop.fields)) {
						return new Error("Required prop '" + propName + "." + optionProp + ".fields' was not specified in '" + componentName + "'");
					}
				}
			}
		})
	}).isRequired,
	schema: _propTypes2.default.shape({
		type: _propTypes2.default.oneOf(["object"])
	}).isRequired,
	formData: _propTypes2.default.object.isRequired
};
exports.default = NestField;
function getPropsForFields(_ref5, fields, title) {
	var schema = _ref5.schema,
	    uiSchema = _ref5.uiSchema,
	    idSchema = _ref5.idSchema,
	    errorSchema = _ref5.errorSchema,
	    formData = _ref5.formData,
	    onChange = _ref5.onChange,
	    definitions = _ref5.registry.definitions;

	var newSchema = { type: "object", properties: {}, title: title };
	var newErrorSchema = {};
	var newFormData = {};
	var newUiSchema = ["classNames"].reduce(function (_uiSchema, prop) {
		if (prop in _uiSchema) _uiSchema[prop] = uiSchema[prop];
		return _uiSchema;
	}, {});
	var fieldsDictionarified = {};

	// For keeping field names compatible with idSchema.
	var flattenPointerName = function flattenPointerName(fieldName) {
		return fieldName[0] === "/" ? fieldName.replace(/(?!^)\//g, "_").substr(1) : fieldName;
	};

	fields.forEach(function (fieldName) {
		[[schema.properties, newSchema.properties, (0, _utils2.schemaJSONPointer)(schema.properties, fieldName)], [uiSchema, newUiSchema, (0, _utils2.uiSchemaJSONPointer)(schema.properties, fieldName)], [errorSchema, newErrorSchema, fieldName], [formData, newFormData, fieldName]].forEach(function (_ref6) {
			var _ref7 = (0, _slicedToArray3.default)(_ref6, 3),
			    originalPropContainer = _ref7[0],
			    newPropContainer = _ref7[1],
			    _fieldName = _ref7[2];

			if (_fieldName !== undefined && (0, _utils2.checkJSONPointer)(originalPropContainer, _fieldName)) {
				newPropContainer[flattenPointerName(fieldName)] = (0, _utils2.parseJSONPointer)(originalPropContainer, _fieldName);
			}
		});
		fieldsDictionarified[fieldName] = true;
	});

	// TODO Doesn't work for JSON Pointer fields.
	if (uiSchema["ui:order"]) newUiSchema["ui:order"] = uiSchema["ui:order"].filter(function (ord) {
		return fieldsDictionarified[ord] || ord === "*";
	});
	if (schema.required) newSchema.required = schema.required.filter(function (req) {
		return fieldsDictionarified[req];
	});

	var newIdSchema = (0, _utils.toIdSchema)(newSchema, idSchema.$id, definitions);

	var newOnChange = function newOnChange(formData) {
		var newFormData = fields.reduce(function (_formData, field) {
			_formData = (0, _utils2.updateSafelyWithJSONPointer)(_formData, _formData[flattenPointerName[field]], field, !!"immutably", function (__formData, path) {
				var _schema = (0, _utils2.parseJSONPointer)(schema, (0, _utils2.schemaJSONPointer)(schema, path));
				return (0, _utils.getDefaultFormState)(_schema, undefined, definitions);
			});
			_formData = (0, _utils2.immutableDelete)(_formData, flattenPointerName(field));
			return _formData;
		}, formData);
		return onChange(newFormData);
	};

	return {
		schema: newSchema,
		uiSchema: newUiSchema,
		idSchema: newIdSchema,
		errorSchema: newErrorSchema,
		formData: newFormData,
		onChange: onChange ? newOnChange : undefined
	};
}