"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _slicedToArray2 = require("babel-runtime/helpers/slicedToArray");

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _extends2 = require("babel-runtime/helpers/extends");

var _extends3 = _interopRequireDefault(_extends2);

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _ObjectField = require("react-jsonschema-form/lib/components/fields/ObjectField");

var _ObjectField2 = _interopRequireDefault(_ObjectField);

var _utils = require("react-jsonschema-form/lib/utils");

var _reactBootstrap = require("react-bootstrap");

var _utils2 = require("../../utils");

var _ArrayFieldTemplate = require("../ArrayFieldTemplate");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (props) {
	var Template = props.uiSchema["ui:grid"] ? GridTemplate : ObjectFieldTemplate;
	var id = (0, _utils2.getUUID)(props.formData);
	var formContext = id ? (0, _extends3.default)({}, props.formContext, { _parentLajiFormId: id }) : props.formContext;
	return _react2.default.createElement(_ObjectField2.default, (0, _extends3.default)({}, props, { registry: (0, _extends3.default)({}, props.registry, { ObjectFieldTemplate: Template, formContext: formContext }), formContext: formContext }));
};

function ObjectFieldTemplate(props) {
	var TitleField = props.TitleField,
	    DescriptionField = props.DescriptionField;


	var buttons = getGlyphButtons(props);

	var _map = ["top", "bottom", "left", "right"].map(function (pos) {
		var buttons = (0, _ArrayFieldTemplate.getButtonsForPosition)(props, (0, _utils2.getUiOptions)(props.uiSchema).buttons, pos, "no default");
		return buttons ? _react2.default.createElement(
			_reactBootstrap.ButtonToolbar,
			{ key: "buttons-" + pos },
			buttons
		) : null;
	}),
	    _map2 = (0, _slicedToArray3.default)(_map, 4),
	    topButtons = _map2[0],
	    bottomButtons = _map2[1],
	    leftButtons = _map2[2],
	    rightButtons = _map2[3];

	var _getClassNames = getClassNames(props, buttons),
	    containerClassName = _getClassNames.containerClassName,
	    schemaClassName = _getClassNames.schemaClassName,
	    buttonsClassName = _getClassNames.buttonsClassName;

	buttons = _react2.default.createElement(
		"div",
		{ className: buttonsClassName },
		buttons
	);

	return _react2.default.createElement(
		"div",
		{ className: containerClassName },
		_react2.default.createElement(
			"fieldset",
			{ className: schemaClassName },
			props.title && _react2.default.createElement(TitleField, {
				id: props.idSchema.$id + "__title",
				title: props.title,
				required: props.required || props.uiSchema["ui:required"],
				formContext: props.formContext,
				className: (0, _utils2.getUiOptions)(props.uiSchema).titleClassName,
				help: props.uiSchema["ui:help"],
				buttons: buttons
			}),
			topButtons,
			props.description && _react2.default.createElement(DescriptionField, {
				id: props.idSchema.$id + "__description",
				description: props.description,
				formContext: props.formContext
			}),
			leftButtons && _react2.default.createElement(
				"div",
				{ className: "pull-left" },
				leftButtons
			),
			props.properties.map(function (_ref) {
				var content = _ref.content;
				return content;
			}),
			rightButtons && _react2.default.createElement(
				"div",
				{ className: "pull-right" },
				rightButtons
			),
			bottomButtons
		),
		!props.title && buttons ? buttons : undefined
	);
}

function GridTemplate(props) {
	var schema = props.schema,
	    uiSchema = props.uiSchema,
	    idSchema = props.idSchema,
	    properties = props.properties,
	    TitleField = props.TitleField;

	var gridOptions = props.uiSchema["ui:grid"] || {};

	var rows = [];
	var lastRow = [];

	var colsToRows = {};
	(gridOptions.rows || []).forEach(function (row, i) {
		row.forEach(function (col) {
			colsToRows[col] = i;
		});
	});

	var _gridOptions$rowTitle = gridOptions.rowTitles,
	    rowTitles = _gridOptions$rowTitle === undefined ? [] : _gridOptions$rowTitle,
	    _gridOptions$classNam = gridOptions.classNames,
	    classNames = _gridOptions$classNam === undefined ? {} : _gridOptions$classNam;
	var Label = props.formContext.Label;


	var addRowTitles = function addRowTitles(rows, rowTitles) {
		for (var i = 0; i < rowTitles.length; i++) {
			rows[i] = [];
			var titleCols = getCols(props, { type: "string" }, uiSchema["rowTitle"], "rowTitle");
			rows[i].push(_react2.default.createElement(
				_reactBootstrap.Col,
				(0, _extends3.default)({}, titleCols, { key: "title_" + i, className: classNames["rowTitle"] }),
				_react2.default.createElement(Label, { id: idSchema.$id + "_row_" + i,
					label: rowTitles[i].title,
					help: rowTitles[i].help })
			));
		}
	};

	function getRow(col, colsToRows, rows) {
		var colRow = colsToRows[col];
		if (colRow !== undefined) {
			if (!rows[colRow]) rows[colRow] = [];
			return rows[colRow];
		} else {
			return lastRow;
		}
	}

	addRowTitles(rows, rowTitles);

	(0, _utils.orderProperties)((0, _keys2.default)(schema.properties), uiSchema["ui:order"]).forEach(function (propertyName) {
		var property = schema.properties[propertyName];

		if (!property) return;

		var uiSchemaProperty = uiSchema[propertyName];
		var cols = getCols(props, property, uiSchemaProperty, propertyName);

		var propertiesByName = properties.reduce(function (propertiesByName, _prop) {
			propertiesByName[_prop.name] = _prop;
			return propertiesByName;
		}, {});

		if (!(0, _utils2.isHidden)(uiSchema, propertyName)) getRow(propertyName, colsToRows, rows).push(_react2.default.createElement(
			_reactBootstrap.Col,
			(0, _extends3.default)({ key: propertyName }, cols, { className: classNames[propertyName] }),
			propertiesByName[propertyName].content
		));
	});

	if (lastRow.length > 0) rows.push(lastRow);

	var title = schema.title;
	var _title = uiSchema["ui:title"];

	var fieldTitle = _title || (title !== undefined ? title : props.name);

	var buttons = getGlyphButtons(props);

	var _map3 = ["top", "bottom", "left", "right"].map(function (pos) {
		var buttons = (0, _ArrayFieldTemplate.getButtonsForPosition)(props, (0, _utils2.getUiOptions)(uiSchema).buttons, pos, "no default");
		return buttons ? _react2.default.createElement(
			_reactBootstrap.ButtonToolbar,
			{ key: "buttons-" + pos },
			(0, _ArrayFieldTemplate.getButtonsForPosition)(props, (0, _utils2.getUiOptions)(uiSchema).buttons, pos, "no default")
		) : null;
	}),
	    _map4 = (0, _slicedToArray3.default)(_map3, 4),
	    topButtons = _map4[0],
	    bottomButtons = _map4[1],
	    leftButtons = _map4[2],
	    rightButtons = _map4[3];

	var _getClassNames2 = getClassNames(props, buttons),
	    containerClassName = _getClassNames2.containerClassName,
	    schemaClassName = _getClassNames2.schemaClassName,
	    buttonsClassName = _getClassNames2.buttonsClassName;

	buttons = _react2.default.createElement(
		"div",
		{ className: buttonsClassName },
		buttons
	);

	return _react2.default.createElement(
		"div",
		{ className: containerClassName },
		_react2.default.createElement(
			"fieldset",
			{ className: schemaClassName },
			!(0, _utils2.isEmptyString)(fieldTitle) ? _react2.default.createElement(TitleField, { title: fieldTitle,
				className: (0, _utils2.getUiOptions)(props.uiSchema).titleClassName,
				buttons: buttons,
				help: uiSchema["ui:help"],
				id: idSchema.$id }) : null,
			topButtons,
			leftButtons && _react2.default.createElement(
				"div",
				{ className: "pull-left" },
				leftButtons
			),
			rows.map(function (row, i) {
				return _react2.default.createElement(
					_reactBootstrap.Row,
					{ key: i },
					row
				);
			}),
			rightButtons && _react2.default.createElement(
				"div",
				{ className: "pull-right" },
				rightButtons
			),
			bottomButtons
		),
		!props.title && buttons ? buttons : null
	);
}

function getCols(props, schema, uiSchema, property) {
	var options = props.uiSchema["ui:grid"];
	var cols = { lg: 12, md: 12, sm: 12, xs: 12 };
	var uiField = uiSchema && uiSchema["ui:field"] ? uiSchema["ui:field"] : undefined;

	(0, _keys2.default)(cols).forEach(function (col) {
		var optionCol = options[col];
		if ((!(0, _utils2.isObject)(optionCol) || !optionCol[property]) && schema.type === "array" && !(schema.items && schema.items.enum && (0, _utils.isMultiSelect)(schema, uiSchema) || uiField === "SingleItemArrayField" || schema.items && schema.items.type === "string" && uiField !== "ImageArrayField" && uiField !== "AudioArrayField") || schema.type === "string" && uiSchema && (0, _utils2.getNestedUiFieldsList)(uiSchema).indexOf("SelectTreeField") !== -1) {
			return cols;
		}
		if ((0, _utils2.isObject)(optionCol)) {
			var selector = undefined;
			if (optionCol[property]) selector = property;else if (optionCol["*"]) selector = "*";
			cols[col] = parseInt(optionCol[selector]);
		} else {
			cols[col] = parseInt(optionCol);
		}
	});

	return cols;
}

function getClassNames(props, buttons) {
	var vertical = props.uiSchema["ui:buttonsVertical"];
	var containerClassName = void 0,
	    schemaClassName = void 0,
	    buttonsClassName = void 0;
	if (buttons && buttons.length) {
		containerClassName = "laji-form-field-template-item" + (vertical ? " keep-vertical" : "");
		schemaClassName = "laji-form-field-template-schema";
		buttonsClassName = "laji-form-field-template-buttons";
		if (props.title) buttonsClassName += " pull-right";
	}
	return { containerClassName: containerClassName, schemaClassName: schemaClassName, buttonsClassName: buttonsClassName };
}

function getGlyphButtons(props) {
	var uiSchema = props.uiSchema;

	var buttons = uiSchema["ui:buttons"] || [];
	var buttonDescriptions = ((0, _utils2.getUiOptions)(uiSchema).buttons || []).filter(function (buttonDef) {
		return !buttonDef.position;
	});
	if (buttonDescriptions) {
		buttons = [].concat((0, _toConsumableArray3.default)(buttons), (0, _toConsumableArray3.default)(buttonDescriptions.map(function (buttonDescription) {
			return (0, _ArrayFieldTemplate.getButton)(buttonDescription, props);
		})));
	}

	return buttons ? props.title ? buttons.reverse() : buttons : null;
}